/* -*- mode: c; c-basic-offset: 4; -*- */
/* vi:set cindent ts=8 sts=4 sw=4: */
#ifndef KEYMAP_H
#define KEYMAP_H

const char *command_reference(int menu);
void get_key_descr_for_cmd(int cmd_id, char *s, int slen);
void key_description(int keymap_index, char *s, int slen);

enum {
    CMD_PREVIOUS_ENTRY,
    CMD_NEXT_ENTRY,
    CMD_MOVE_SONG_FIRST,
    CMD_MOVE_SONG_LAST,
    CMD_QUIT,
    CMD_ADD_DIRECTORY,
    CMD_CLEAR_PLAYLIST,
    CMD_FIRST_ENTRY,
    CMD_LAST_ENTRY,
    CMD_SCROLL_UP,
    CMD_SCROLL_DOWN,
    CMD_PLAY_SONG,
    CMD_REDRAW_SCREEN,
    CMD_TOP_OF_PAGE,
    CMD_MIDDLE_OF_PAGE,
    CMD_BOTTOM_OF_PAGE,
    CMD_PAGE_UP,
    CMD_PAGE_DOWN,
    CMD_HALF_PAGE_UP,
    CMD_HALF_PAGE_DOWN,
    CMD_PLAY_TOGGLED_SONGS,
    CMD_APPEND_PLAYLIST,
    CMD_MOVE_SONG_TO_TOP_OF_PAGE,
    CMD_MOVE_SONG_TO_BOTTOM_OF_PAGE,
    CMD_MOVE_SONG_TO_MIDDLE_OF_PAGE,
    CMD_TOGGLE_SONG,
    CMD_TOGGLE_CONTINUOUS_FLAG,
    CMD_TOGGLE_RANDOM_FLAG,
    CMD_ADD_SONG,
    CMD_REMOVE_SONG,
    CMD_MOVE_SONG_UP,
    CMD_MOVE_SONG_DOWN,
    CMD_HELP,
    CMD_GO_TO,
    CMD_SAVE_PLAYLIST,
    CMD_SAVE_PLAYLIST_AS,
    CMD_OPTIONS,
    CMD_CHANGE_DIRECTORY,
    CMD_WRITE_SONG_NAMES,
    CMD_PREVIOUS_MENU,
    CMD_TOGGLE_OPTION,
    CMD_RESET_ALL_OPTIONS,
    CMD_EDIT_OPTION,
    CMD_BROWSE_DIRECTORY,
    CMD_SORT_REVERSE,
    CMD_SORT,
    CMD_MOVE_SONG_TO,
    CMD_ADD_CURRENT_SONG,
    CMD_ADD_CURRENT_DIRECTORY,
    CMD_REGEXP_SEARCH_FORWARD,
    CMD_REGEXP_SEARCH_BACKWARD,
    CMD_REPEAT_LAST_SEARCH_OPPOSITE,
    CMD_REPEAT_LAST_SEARCH,
    CMD_OPEN_PLAYLIST,
    CMD_TOGGLE_ALL_SONGS_OFF,
    CMD_TOGGLE_ALL_SONGS_ON,
    CMD_ABORT,
    CMD_BEGINNING_OF_LINE,
    CMD_END_OF_LINE,
    CMD_FORWARD_CHAR,
    CMD_BACKWARD_CHAR,
    CMD_DELETE_CHAR,
    CMD_KILL_WHOLE_LINE,
    CMD_KILL_LINE,
    CMD_BACKWARD_KILL_WORD,
    CMD_BACKWARD_DELETE_CHAR,
    CMD_ENTER,
    CMD_BROWSE,

    KEYMAP_TERMINATOR
};

/* Command types */
enum {MOTION, SONG_MOVEMENT, SCROLLING, PLAYING, SEARCHING, MISC};

/* Flags for the keymap entries */
/* The command can have a numeric argument */
#define NUM            (1<<0)

/* Maximum length of command names */
#define MAX_CMD_NAME_LEN 27

struct keymap_entry {
    const int firstchar; /* first character of command */
    const int nextchar;  /* next character (optionally, set to 0 if none) */
    const int flags;
    const int menus;     /* menus in which the command can be used */
    const int type;      /* the type of command */
    const char *name;    /* name of command, max length `MAX_CMD_NAME_LEN' */
    const int cmd_id;    /* unique id for the command */
    const char *descr;   /* description of the command */
};

extern struct keymap_entry keymap[];

#endif /* KEYMAP_H */
